@@ -1,39 +1,44 @@
-#
-# $Id: MemberMixin.pm,v 1.3 1996/02/26 19:12:24 aas Exp $
-
 package LWP::MemberMixin;
 
+sub _elem
+{
+    my $self = shift;
+    my $elem = shift;
+    my $old = $self->{$elem};
+    $self->{$elem} = shift if @_;
+    return $old;
+}
+
+1;
+
+__END__
+
 =head1 NAME
 
 LWP::MemberMixin - Member access mixin class
 
+=head1 SYNOPSIS
+
+ package Foo;
+ require LWP::MemberMixin;
+ @ISA=qw(LWP::MemberMixin);
+
 =head1 DESCRIPTION
 
 A mixin class to get methods that provide easy access to member
 variables in the %$self.
+Ideally there should be better Perl language support for this.
 
-=head1 BUGS
+There is only one method provided:
 
-Ideally there should be better Perl langauge support for this.
+=over 4
 
-=head1 METHODS
-
-=head2 _elem($elem [, $val])
+=item _elem($elem [, $val])
 
 Internal method to get/set the value of member variable
-C<$elem>. If C<$val> is defined it is used as the new value
-for the member variable.  If it is undefined the current
+C<$elem>. If C<$val> is present it is used as the new value
+for the member variable.  If it is not present the current
 value is not touched. In both cases the previous value of
 the member variable is returned.
 
-=cut
-
-sub _elem
-{
-    my($self, $elem, $val) = @_;
-    my $old = $self->{$elem};
-    $self->{$elem} = $val if defined $val;
-    return $old;
-}
-
-1;
+=back
