@@ -39,55 +39,55 @@ sub request
     # check proxy
     if (defined $proxy)
     {
-	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
-				  'You can not proxy with mail';
+	return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST,
+				  'You can not proxy with mail');
     }
 
     # check method
     my $method = $request->method;
 
     if ($method ne 'POST') {
-	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
+	return HTTP::Response->new( &HTTP::Status::RC_BAD_REQUEST,
 				  'Library does not allow method ' .
-				  "$method for 'mailto:' URLs";
+				  "$method for 'mailto:' URLs");
     }
 
     # check url
-    my $url = $request->url;
+    my $url = $request->uri;
 
     my $scheme = $url->scheme;
     if ($scheme ne 'mailto') {
-	return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-			 "LWP::Protocol::mailto::request called for '$scheme'";
+	return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+			 "LWP::Protocol::mailto::request called for '$scheme'");
     }
     if ($^O eq "MacOS") {
 	eval {
 	    require Mail::Internet;
 	};
 	if($@) {
-	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-	               "You don't have MailTools installed";
+	    return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+	               "You don't have MailTools installed");
 	}
 	unless ($ENV{SMTPHOSTS}) {
-	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-	               "You don't have SMTPHOSTS defined";
+	    return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+	               "You don't have SMTPHOSTS defined");
 	}
     }
     else {
 	unless (-x $SENDMAIL) {
-	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-	               "You don't have $SENDMAIL";
+	    return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+	               "You don't have $SENDMAIL");
     }
     }
     if ($^O eq "MacOS") {
 	    $mail = Mail::Internet->new or
-	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-	    "Can't get a Mail::Internet object";
+	    return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+	    "Can't get a Mail::Internet object");
     }
     else {
 	open(SENDMAIL, "| $SENDMAIL -oi -t") or
-	    return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-	               "Can't run $SENDMAIL: $!";
+	    return HTTP::Response->new( &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+	               "Can't run $SENDMAIL: $!");
     }
     if ($^O eq "MacOS") {
 	$addr = $url->encoded822addr;
